@@ -246,10 +246,11 @@ class RawExtrinsicMetadataRow(BaseRow):
         "authority_type",
         "authority_url",
         "discovery_date",
-        "fetcher_name",
-        "fetcher_version",
+        "id",
     )
 
+    id: bytes
+
     type: str
     target: str
 
@@ -280,3 +281,14 @@ class ObjectCountRow(BaseRow):
     partition_key: int
     object_type: str
     count: int
+
+
+@dataclasses.dataclass
+class ExtIDRow(BaseRow):
+    TABLE = "extid"
+    PARTITION_KEY = ("target", "target_type", "extid", "extid_type")
+
+    extid_type: str
+    extid: bytes
+    target_type: str
+    target: bytes
