@@ -24,17 +24,19 @@ from typing import (
 
 import attr
 
+from swh.core.api.classes import stream_results
 from swh.core.api.serializers import msgpack_dumps, msgpack_loads
 from swh.model.hashutil import DEFAULT_ALGORITHMS
-from swh.model.identifiers import SWHID, parse_swhid
+from swh.model.identifiers import CoreSWHID, ExtendedSWHID
+from swh.model.identifiers import ObjectType as SwhidObjectType
 from swh.model.model import (
     Content,
     Directory,
     DirectoryEntry,
+    ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
     Origin,
     OriginVisit,
     OriginVisitStatus,
@@ -60,12 +62,14 @@ from swh.storage.writer import JournalWriter
 
 from . import converters
 from ..exc import HashCollision, StorageArgumentException
-from .common import TOKEN_BEGIN, TOKEN_END, hash_url, remove_keys
+from ..utils import remove_keys
+from .common import TOKEN_BEGIN, TOKEN_END, hash_url
 from .cql import CqlRunner
 from .model import (
     ContentRow,
     DirectoryEntryRow,
     DirectoryRow,
+    ExtIDRow,
     MetadataAuthorityRow,
     MetadataFetcherRow,
     OriginRow,
@@ -118,20 +122,22 @@ class CassandraStorage:
             c for c in contents if not self._cql_runner.content_get_from_pk(c.to_dict())
         ]
 
-        self.journal_writer.content_add(contents)
-
         if with_data:
             # First insert to the objstorage, if the endpoint is
             # `content_add` (as opposed to `content_add_metadata`).
-            # TODO: this should probably be done in concurrently to inserting
-            # in index tables (but still before the main table; so an entry is
-            # only added to the main table after everything else was
-            # successfully inserted.
+
+            # Must add to the objstorage before the DB and journal. Otherwise:
+            # 1. in case of a crash the DB may "believe" we have the content, but
+            #    we didn't have time to write to the objstorage before the crash
+            # 2. the objstorage mirroring, which reads from the journal, may attempt to
+            #    read from the objstorage before we finished writing it
             summary = self.objstorage.content_add(
                 c for c in contents if c.status != "absent"
             )
             content_add_bytes = summary["content:add:bytes"]
 
+        self.journal_writer.content_add(contents)
+
         content_add = 0
         for content in contents:
             content_add += 1
@@ -663,13 +669,16 @@ class CassandraStorage:
         }
 
     def snapshot_count_branches(
-        self, snapshot_id: Sha1Git
+        self, snapshot_id: Sha1Git, branch_name_exclude_prefix: Optional[bytes] = None,
     ) -> Optional[Dict[Optional[str], int]]:
         if self._cql_runner.snapshot_missing([snapshot_id]):
             # Makes sure we don't fetch branches for a snapshot that is
             # being added.
             return None
-        return self._cql_runner.snapshot_count_branches(snapshot_id)
+
+        return self._cql_runner.snapshot_count_branches(
+            snapshot_id, branch_name_exclude_prefix
+        )
 
     def snapshot_get_branches(
         self,
@@ -677,6 +686,8 @@ class CassandraStorage:
         branches_from: bytes = b"",
         branches_count: int = 1000,
         target_types: Optional[List[str]] = None,
+        branch_name_include_substring: Optional[bytes] = None,
+        branch_name_exclude_prefix: Optional[bytes] = None,
     ) -> Optional[PartialBranches]:
         if self._cql_runner.snapshot_missing([snapshot_id]):
             # Makes sure we don't fetch branches for a snapshot that is
@@ -687,7 +698,10 @@ class CassandraStorage:
         while len(branches) < branches_count + 1:
             new_branches = list(
                 self._cql_runner.snapshot_branch_get(
-                    snapshot_id, branches_from, branches_count + 1
+                    snapshot_id,
+                    branches_from,
+                    branches_count + 1,
+                    branch_name_exclude_prefix,
                 )
             )
 
@@ -706,6 +720,18 @@ class CassandraStorage:
                     if branch.target is not None and branch.target_type in target_types
                 ]
 
+            # Filter by branches_name_pattern
+            if branch_name_include_substring:
+                new_branches_filtered = [
+                    branch
+                    for branch in new_branches_filtered
+                    if branch.name is not None
+                    and (
+                        branch_name_include_substring is None
+                        or branch_name_include_substring in branch.name
+                    )
+                ]
+
             branches.extend(new_branches_filtered)
 
             if len(new_branches) < branches_count + 1:
@@ -818,6 +844,7 @@ class CassandraStorage:
         limit: int = 50,
         regexp: bool = False,
         with_visit: bool = False,
+        visit_types: Optional[List[str]] = None,
     ) -> PagedResult[Origin]:
         # TODO: remove this endpoint, swh-search should be used instead.
         next_page_token = None
@@ -833,6 +860,18 @@ class CassandraStorage:
         if with_visit:
             origin_rows = [row for row in origin_rows if row.next_visit_id > 1]
 
+        if visit_types:
+
+            def _has_visit_types(origin, visit_types):
+                for origin_visit in stream_results(self.origin_visit_get, origin):
+                    if origin_visit.type in visit_types:
+                        return True
+                return False
+
+            origin_rows = [
+                row for row in origin_rows if _has_visit_types(row.url, visit_types)
+            ]
+
         origins = [Origin(url=row.url) for row in origin_rows]
 
         origins = origins[offset : offset + limit + 1]
@@ -899,12 +938,15 @@ class CassandraStorage:
     def _origin_visit_status_add(self, visit_status: OriginVisitStatus) -> None:
         """Add an origin visit status"""
         if visit_status.type is None:
-            origin_row = self._cql_runner.origin_visit_get_one(
+            visit_row = self._cql_runner.origin_visit_get_one(
                 visit_status.origin, visit_status.visit
             )
-            if origin_row is None:
-                raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
-            visit_status = attr.evolve(visit_status, type=origin_row.type)
+            if visit_row is None:
+                raise StorageArgumentException(
+                    f"Unknown origin visit {visit_status.visit} "
+                    f"of origin {visit_status.origin}"
+                )
+            visit_status = attr.evolve(visit_status, type=visit_row.type)
 
         self.journal_writer.origin_visit_status_add([visit_status])
         self._cql_runner.origin_visit_status_add_one(
@@ -1157,7 +1199,8 @@ class CassandraStorage:
 
             try:
                 row = RawExtrinsicMetadataRow(
-                    type=metadata_entry.type.value,
+                    id=metadata_entry.id,
+                    type=metadata_entry.target.object_type.name.lower(),
                     target=str(metadata_entry.target),
                     authority_type=metadata_entry.authority.type.value,
                     authority_url=metadata_entry.authority.url,
@@ -1180,41 +1223,20 @@ class CassandraStorage:
 
     def raw_extrinsic_metadata_get(
         self,
-        type: MetadataTargetType,
-        target: Union[str, SWHID],
+        target: ExtendedSWHID,
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
     ) -> PagedResult[RawExtrinsicMetadata]:
-        if type == MetadataTargetType.ORIGIN:
-            if isinstance(target, SWHID):
-                raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with type='origin', "
-                    f"but provided target is a SWHID: {target!r}"
-                )
-        else:
-            if not isinstance(target, SWHID):
-                raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with type!='origin', "
-                    f"but provided target is not a SWHID: {target!r}"
-                )
-
         if page_token is not None:
-            (after_date, after_fetcher_name, after_fetcher_url) = msgpack_loads(
-                base64.b64decode(page_token)
-            )
+            (after_date, id_) = msgpack_loads(base64.b64decode(page_token))
             if after and after_date < after:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._cql_runner.raw_extrinsic_metadata_get_after_date_and_fetcher(  # noqa
-                str(target),
-                authority.type.value,
-                authority.url,
-                after_date,
-                after_fetcher_name,
-                after_fetcher_url,
+            entries = self._cql_runner.raw_extrinsic_metadata_get_after_date_and_id(
+                str(target), authority.type.value, authority.url, after_date, id_,
             )
         elif after is not None:
             entries = self._cql_runner.raw_extrinsic_metadata_get_after_date(
@@ -1235,7 +1257,6 @@ class CassandraStorage:
             assert str(target) == entry.target
 
             result = RawExtrinsicMetadata(
-                type=MetadataTargetType(entry.type),
                 target=target,
                 authority=MetadataAuthority(
                     type=MetadataAuthorityType(entry.authority_type),
@@ -1249,11 +1270,11 @@ class CassandraStorage:
                 metadata=entry.metadata,
                 origin=entry.origin,
                 visit=entry.visit,
-                snapshot=map_optional(parse_swhid, entry.snapshot),
-                release=map_optional(parse_swhid, entry.release),
-                revision=map_optional(parse_swhid, entry.revision),
+                snapshot=map_optional(CoreSWHID.from_string, entry.snapshot),
+                release=map_optional(CoreSWHID.from_string, entry.release),
+                revision=map_optional(CoreSWHID.from_string, entry.revision),
                 path=entry.path,
-                directory=map_optional(parse_swhid, entry.directory),
+                directory=map_optional(CoreSWHID.from_string, entry.directory),
             )
 
             results.append(result)
@@ -1263,13 +1284,7 @@ class CassandraStorage:
             assert len(results) == limit
             last_result = results[-1]
             next_page_token: Optional[str] = base64.b64encode(
-                msgpack_dumps(
-                    (
-                        last_result.discovery_date,
-                        last_result.fetcher.name,
-                        last_result.fetcher.version,
-                    )
-                )
+                msgpack_dumps((last_result.discovery_date, last_result.id,))
             ).decode()
         else:
             next_page_token = None
@@ -1324,6 +1339,70 @@ class CassandraStorage:
         else:
             return None
 
+    # ExtID tables
+    def extid_add(self, ids: List[ExtID]) -> Dict[str, int]:
+        extids = [
+            extid
+            for extid in ids
+            if not self._cql_runner.extid_get_from_pk(
+                extid_type=extid.extid_type, extid=extid.extid, target=extid.target,
+            )
+        ]
+
+        self.journal_writer.extid_add(extids)
+
+        inserted = 0
+        for extid in extids:
+            extidrow = ExtIDRow(
+                extid_type=extid.extid_type,
+                extid=extid.extid,
+                target_type=extid.target.object_type.value,
+                target=extid.target.object_id,
+            )
+            (token, insertion_finalizer) = self._cql_runner.extid_add_prepare(extidrow)
+            self._cql_runner.extid_index_add_one(extidrow, token)
+            insertion_finalizer()
+            inserted += 1
+        return {"extid:add": inserted}
+
+    def extid_get_from_extid(self, id_type: str, ids: List[bytes]) -> List[ExtID]:
+        result: List[ExtID] = []
+        for extid in ids:
+            extidrows = list(self._cql_runner.extid_get_from_extid(id_type, extid))
+            result.extend(
+                ExtID(
+                    extid_type=extidrow.extid_type,
+                    extid=extidrow.extid,
+                    target=CoreSWHID(
+                        object_type=extidrow.target_type, object_id=extidrow.target,
+                    ),
+                )
+                for extidrow in extidrows
+            )
+        return result
+
+    def extid_get_from_target(
+        self, target_type: SwhidObjectType, ids: List[Sha1Git]
+    ) -> List[ExtID]:
+        result: List[ExtID] = []
+        for target in ids:
+            extidrows = list(
+                self._cql_runner.extid_get_from_target(target_type.value, target)
+            )
+            result.extend(
+                ExtID(
+                    extid_type=extidrow.extid_type,
+                    extid=extidrow.extid,
+                    target=CoreSWHID(
+                        object_type=SwhidObjectType(extidrow.target_type),
+                        object_id=extidrow.target,
+                    ),
+                )
+                for extidrow in extidrows
+            )
+        return result
+
+    # Misc
     def clear_buffers(self, object_types: Sequence[str] = ()) -> None:
         """Do nothing
 
