@@ -21,6 +21,7 @@ from typing import (
     Union,
 )
 
+from swh.model.identifiers import ExtendedSWHID
 from swh.model.model import Content, Sha1Git, SkippedContent
 from swh.storage.cassandra import CassandraStorage
 from swh.storage.cassandra.model import (
@@ -28,6 +29,7 @@ from swh.storage.cassandra.model import (
     ContentRow,
     DirectoryEntryRow,
     DirectoryRow,
+    ExtIDRow,
     MetadataAuthorityRow,
     MetadataFetcherRow,
     ObjectCountRow,
@@ -164,6 +166,7 @@ class InMemoryCqlRunner:
         self._metadata_authorities = Table(MetadataAuthorityRow)
         self._metadata_fetchers = Table(MetadataFetcherRow)
         self._raw_extrinsic_metadata = Table(RawExtrinsicMetadataRow)
+        self._extid = Table(ExtIDRow)
         self._stat_counters = defaultdict(int)
 
     def increment_counter(self, object_type: str, nb: int):
@@ -386,11 +389,17 @@ class InMemoryCqlRunner:
     def snapshot_branch_add_one(self, branch: SnapshotBranchRow) -> None:
         self._snapshot_branches.insert(branch)
 
-    def snapshot_count_branches(self, snapshot_id: Sha1Git) -> Dict[Optional[str], int]:
+    def snapshot_count_branches(
+        self, snapshot_id: Sha1Git, branch_name_exclude_prefix: Optional[bytes] = None,
+    ) -> Dict[Optional[str], int]:
         """Returns a dictionary from type names to the number of branches
         of that type."""
         counts: Dict[Optional[str], int] = defaultdict(int)
         for branch in self._snapshot_branches.get_from_partition_key((snapshot_id,)):
+            if branch_name_exclude_prefix and branch.name.startswith(
+                branch_name_exclude_prefix
+            ):
+                continue
             if branch.target_type is None:
                 target_type = None
             else:
@@ -399,11 +408,18 @@ class InMemoryCqlRunner:
         return counts
 
     def snapshot_branch_get(
-        self, snapshot_id: Sha1Git, from_: bytes, limit: int
+        self,
+        snapshot_id: Sha1Git,
+        from_: bytes,
+        limit: int,
+        branch_name_exclude_prefix: Optional[bytes] = None,
     ) -> Iterable[SnapshotBranchRow]:
         count = 0
         for branch in self._snapshot_branches.get_from_partition_key((snapshot_id,)):
-            if branch.name >= from_:
+            prefix = branch_name_exclude_prefix
+            if branch.name >= from_ and (
+                prefix is None or not branch.name.startswith(prefix)
+            ):
                 count += 1
                 yield branch
             if count >= limit:
@@ -583,23 +599,22 @@ class InMemoryCqlRunner:
         )
         return (m for m in metadata if m.discovery_date > after)
 
-    def raw_extrinsic_metadata_get_after_date_and_fetcher(
+    def raw_extrinsic_metadata_get_after_date_and_id(
         self,
         target: str,
         authority_type: str,
         authority_url: str,
         after_date: datetime.datetime,
-        after_fetcher_name: str,
-        after_fetcher_version: str,
+        after_id: bytes,
     ) -> Iterable[RawExtrinsicMetadataRow]:
         metadata = self._raw_extrinsic_metadata.get_from_partition_key((target,))
-        after_tuple = (after_date, after_fetcher_name, after_fetcher_version)
+        after_tuple = (after_date, after_id)
         return (
             m
             for m in metadata
             if m.authority_type == authority_type
             and m.authority_url == authority_url
-            and (m.discovery_date, m.fetcher_name, m.fetcher_version) > after_tuple
+            and (m.discovery_date, m.id) > after_tuple
         )
 
     def raw_extrinsic_metadata_get(
@@ -612,6 +627,49 @@ class InMemoryCqlRunner:
             if m.authority_type == authority_type and m.authority_url == authority_url
         )
 
+    #########################
+    # 'extid' table
+    #########################
+    def _extid_add_finalize(self, extid: ExtIDRow) -> None:
+        self._extid.insert(extid)
+        self.increment_counter("extid", 1)
+
+    def extid_add_prepare(self, extid: ExtIDRow):
+        finalizer = functools.partial(self._extid_add_finalize, extid)
+        return (self._extid.token(self._extid.partition_key(extid)), finalizer)
+
+    def extid_index_add_one(self, extid: ExtIDRow, token: int) -> None:
+        pass
+
+    def extid_get_from_pk(
+        self, extid_type: str, extid: bytes, target: ExtendedSWHID,
+    ) -> Optional[ExtIDRow]:
+        primary_key = self._extid.primary_key_from_dict(
+            dict(
+                extid_type=extid_type,
+                extid=extid,
+                target_type=target.object_type.value,
+                target=target.object_id,
+            )
+        )
+        return self._extid.get_from_primary_key(primary_key)
+
+    def extid_get_from_extid(self, extid_type: str, extid: bytes) -> Iterable[ExtIDRow]:
+        return (
+            row
+            for pk, row in self._extid.iter_all()
+            if row.extid_type == extid_type and row.extid == extid
+        )
+
+    def extid_get_from_target(
+        self, target_type: str, target: bytes
+    ) -> Iterable[ExtIDRow]:
+        return (
+            row
+            for pk, row in self._extid.iter_all()
+            if row.target_type == target_type and row.target == target
+        )
+
 
 class InMemoryStorage(CassandraStorage):
     _cql_runner: InMemoryCqlRunner  # type: ignore
