@@ -5,20 +5,20 @@
 
 import datetime
 from enum import Enum
-from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar
 
 from typing_extensions import Protocol, TypedDict, runtime_checkable
 
 from swh.core.api import remote_api_endpoint
 from swh.core.api.classes import PagedResult as CorePagedResult
-from swh.model.identifiers import SWHID
+from swh.model.identifiers import ExtendedSWHID, ObjectType
 from swh.model.model import (
     Content,
     Directory,
+    ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
     Origin,
     OriginVisit,
     OriginVisitStatus,
@@ -501,6 +501,50 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("extid/from_extid")
+    def extid_get_from_extid(self, id_type: str, ids: List[bytes]) -> List[ExtID]:
+        """Get ExtID objects from external IDs
+
+        Args:
+            id_type: type of the given external identifiers (e.g. 'mercurial')
+            ids: list of external IDs
+
+        Returns:
+            list of ExtID objects
+
+        """
+        ...
+
+    @remote_api_endpoint("extid/from_target")
+    def extid_get_from_target(
+        self, target_type: ObjectType, ids: List[Sha1Git]
+    ) -> List[ExtID]:
+        """Get ExtID objects from target IDs and target_type
+
+        Args:
+            target_type: type the SWH object
+            ids: list of target IDs
+
+        Returns:
+            list of ExtID objects
+
+        """
+        ...
+
+    @remote_api_endpoint("extid/add")
+    def extid_add(self, ids: List[ExtID]) -> Dict[str, int]:
+        """Add a series of ExtID objects
+
+        Args:
+            ids: list of ExtID objects
+
+        Returns:
+            Summary dict of keys with associated count as values
+
+                extid:add: New ExtID objects actually stored in db
+        """
+        ...
+
     @remote_api_endpoint("revision/log")
     def revision_log(
         self, revisions: List[Sha1Git], limit: Optional[int] = None
@@ -681,12 +725,14 @@ class StorageInterface(Protocol):
 
     @remote_api_endpoint("snapshot/count_branches")
     def snapshot_count_branches(
-        self, snapshot_id: Sha1Git
+        self, snapshot_id: Sha1Git, branch_name_exclude_prefix: Optional[bytes] = None,
     ) -> Optional[Dict[Optional[str], int]]:
         """Count the number of branches in the snapshot with the given id
 
         Args:
             snapshot_id: snapshot identifier
+            branch_name_exclude_prefix: if provided, do not count branches whose name
+                starts with given prefix
 
         Returns:
             A dict whose keys are the target types of branches and values their
@@ -702,6 +748,8 @@ class StorageInterface(Protocol):
         branches_from: bytes = b"",
         branches_count: int = 1000,
         target_types: Optional[List[str]] = None,
+        branch_name_include_substring: Optional[bytes] = None,
+        branch_name_exclude_prefix: Optional[bytes] = None,
     ) -> Optional[PartialBranches]:
         """Get the content, possibly partial, of a snapshot with the given id
 
@@ -718,6 +766,10 @@ class StorageInterface(Protocol):
                 target types of branch to return (possible values that can be
                 contained in that list are `'content', 'directory',
                 'revision', 'release', 'snapshot', 'alias'`)
+            branch_name_include_substring: if provided, only return branches whose name
+                contains given substring
+            branch_name_exclude_prefix: if provided, do not return branches whose name
+                contains given prefix
 
         Returns:
             dict: None if the snapshot does not exist;
@@ -1003,6 +1055,7 @@ class StorageInterface(Protocol):
         limit: int = 50,
         regexp: bool = False,
         with_visit: bool = False,
+        visit_types: Optional[List[str]] = None,
     ) -> PagedResult[Origin]:
         """Search for origins whose urls contain a provided string pattern
         or match a provided regular expression.
@@ -1015,6 +1068,8 @@ class StorageInterface(Protocol):
             regexp: if True, consider the provided pattern as a regular
                 expression and return origins whose urls match it
             with_visit: if True, filter out origins with no visit
+            visit_types: Only origins having any of the provided visit types
+                (e.g. git, svn, pypi) will be returned
 
         Yields:
             PagedResult of Origin
@@ -1096,8 +1151,7 @@ class StorageInterface(Protocol):
     @remote_api_endpoint("raw_extrinsic_metadata/get")
     def raw_extrinsic_metadata_get(
         self,
-        type: MetadataTargetType,
-        target: Union[str, SWHID],
+        target: ExtendedSWHID,
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
@@ -1106,8 +1160,7 @@ class StorageInterface(Protocol):
         """Retrieve list of all raw_extrinsic_metadata entries for the id
 
         Args:
-            type: one of the values of swh.model.model.MetadataTargetType
-            target: an URL if type is 'origin', else a core SWHID
+            target: the SWHID of the objects to find metadata on
             authority: a dict containing keys `type` and `url`.
             after: minimum discovery_date for a result to be returned
             page_token: opaque token, used to get the next page of results
