@@ -5,14 +5,17 @@
 
 import datetime
 from typing import Any, Dict, Optional
+import warnings
 
 from swh.core.utils import encode_with_unescape
-from swh.model.identifiers import parse_swhid
+from swh.model.identifiers import CoreSWHID, ExtendedSWHID
+from swh.model.identifiers import ObjectType as SwhidObjectType
+from swh.model.identifiers import origin_identifier
 from swh.model.model import (
+    ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
     ObjectType,
     Person,
     RawExtrinsicMetadata,
@@ -295,13 +298,15 @@ def db_to_release(db_release: Dict[str, Any]) -> Optional[Release]:
 
 
 def db_to_raw_extrinsic_metadata(row) -> RawExtrinsicMetadata:
-    type_ = MetadataTargetType(row["raw_extrinsic_metadata.type"])
     target = row["raw_extrinsic_metadata.target"]
-    if type_ != MetadataTargetType.ORIGIN:
-        target = parse_swhid(target)
+    if not target.startswith("swh:1:"):
+        warnings.warn(
+            "Fetching raw_extrinsic_metadata row with URL target", DeprecationWarning
+        )
+        target = "swh:1:ori:" + origin_identifier({"url": target})
+
     return RawExtrinsicMetadata(
-        type=type_,
-        target=target,
+        target=ExtendedSWHID.from_string(target),
         authority=MetadataAuthority(
             type=MetadataAuthorityType(row["metadata_authority.type"]),
             url=row["metadata_authority.url"],
@@ -314,9 +319,20 @@ def db_to_raw_extrinsic_metadata(row) -> RawExtrinsicMetadata:
         metadata=row["raw_extrinsic_metadata.metadata"],
         origin=row["origin"],
         visit=row["visit"],
-        snapshot=map_optional(parse_swhid, row["snapshot"]),
-        release=map_optional(parse_swhid, row["release"]),
-        revision=map_optional(parse_swhid, row["revision"]),
+        snapshot=map_optional(CoreSWHID.from_string, row["snapshot"]),
+        release=map_optional(CoreSWHID.from_string, row["release"]),
+        revision=map_optional(CoreSWHID.from_string, row["revision"]),
         path=row["path"],
-        directory=map_optional(parse_swhid, row["directory"]),
+        directory=map_optional(CoreSWHID.from_string, row["directory"]),
+    )
+
+
+def db_to_extid(row) -> ExtID:
+    return ExtID(
+        extid=row["extid"],
+        extid_type=row["extid_type"],
+        target=CoreSWHID(
+            object_id=row["target"],
+            object_type=SwhidObjectType[row["target_type"].upper()],
+        ),
     )
