@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -9,7 +9,7 @@ import contextlib
 from contextlib import contextmanager
 import datetime
 import itertools
-from typing import Any, Counter, Dict, Iterable, List, Optional, Sequence, Tuple, Union
+from typing import Any, Counter, Dict, Iterable, List, Optional, Sequence, Tuple
 
 import attr
 import psycopg2
@@ -19,15 +19,15 @@ import psycopg2.pool
 from swh.core.api.serializers import msgpack_dumps, msgpack_loads
 from swh.core.db.common import db_transaction, db_transaction_generator
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
-from swh.model.identifiers import SWHID
+from swh.model.identifiers import ExtendedObjectType, ExtendedSWHID, ObjectType
 from swh.model.model import (
     SHA1_SIZE,
     Content,
     Directory,
+    ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
     Origin,
     OriginVisit,
     OriginVisitStatus,
@@ -215,6 +215,11 @@ class Storage:
 
         contents = [attr.evolve(c, ctime=ctime) for c in content]
 
+        # Must add to the objstorage before the DB and journal. Otherwise:
+        # 1. in case of a crash the DB may "believe" we have the content, but
+        #    we didn't have time to write to the objstorage before the crash
+        # 2. the objstorage mirroring, which reads from the journal, may attempt to
+        #    read from the objstorage before we finished writing it
         objstorage_summary = self.objstorage.content_add(contents)
 
         with self.db() as db:
@@ -634,6 +639,49 @@ class Storage:
     def revision_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.revision_get_random(cur)
 
+    @timed
+    @db_transaction()
+    def extid_get_from_extid(
+        self, id_type: str, ids: List[bytes], db=None, cur=None
+    ) -> List[ExtID]:
+        extids = []
+        for row in db.extid_get_from_extid_list(id_type, ids, cur):
+            if row[0] is not None:
+                extids.append(converters.db_to_extid(dict(zip(db.extid_cols, row))))
+        return extids
+
+    @timed
+    @db_transaction()
+    def extid_get_from_target(
+        self, target_type: ObjectType, ids: List[Sha1Git], db=None, cur=None
+    ) -> List[ExtID]:
+        extids = []
+        for row in db.extid_get_from_swhid_list(target_type.value, ids, cur):
+            if row[0] is not None:
+                extids.append(converters.db_to_extid(dict(zip(db.extid_cols, row))))
+        return extids
+
+    @timed
+    @db_transaction()
+    def extid_add(self, ids: List[ExtID], db=None, cur=None) -> Dict[str, int]:
+        extid = [
+            {
+                "extid": extid.extid,
+                "extid_type": extid.extid_type,
+                "target": extid.target.object_id,
+                "target_type": extid.target.object_type.name.lower(),  # arghh
+            }
+            for extid in ids
+        ]
+        db.mktemp("extid", cur)
+
+        db.copy_to(extid, "tmp_extid", db.extid_cols, cur)
+
+        # move metadata in place
+        db.extid_add_from_temp(cur)
+
+        return {"extid:add": len(extid)}
+
     @timed
     @process_metrics
     @db_transaction()
@@ -755,9 +803,20 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=2000)
     def snapshot_count_branches(
-        self, snapshot_id: Sha1Git, db=None, cur=None
+        self,
+        snapshot_id: Sha1Git,
+        branch_name_exclude_prefix: Optional[bytes] = None,
+        db=None,
+        cur=None,
     ) -> Optional[Dict[Optional[str], int]]:
-        return dict([bc for bc in db.snapshot_count_branches(snapshot_id, cur)])
+        return dict(
+            [
+                bc
+                for bc in db.snapshot_count_branches(
+                    snapshot_id, branch_name_exclude_prefix, cur,
+                )
+            ]
+        )
 
     @timed
     @db_transaction(statement_timeout=2000)
@@ -767,6 +826,8 @@ class Storage:
         branches_from: bytes = b"",
         branches_count: int = 1000,
         target_types: Optional[List[str]] = None,
+        branch_name_include_substring: Optional[bytes] = None,
+        branch_name_exclude_prefix: Optional[bytes] = None,
         db=None,
         cur=None,
     ) -> Optional[PartialBranches]:
@@ -780,8 +841,13 @@ class Storage:
             db.snapshot_get_by_id(
                 snapshot_id,
                 branches_from=branches_from,
-                branches_count=branches_count + 1,
+                # the underlying SQL query can be quite expensive to execute for small
+                # branches_count value, so we ensure a minimum branches limit of 10 for
+                # optimal performances
+                branches_count=max(branches_count + 1, 10),
                 target_types=target_types,
+                branch_name_include_substring=branch_name_include_substring,
+                branch_name_exclude_prefix=branch_name_exclude_prefix,
                 cur=cur,
             )
         )
@@ -800,7 +866,9 @@ class Storage:
             branches[name] = branch
 
         if len(fetched_branches) > branches_count:
-            next_branch = dict(zip(db.snapshot_get_cols, fetched_branches[-1]))["name"]
+            next_branch = dict(
+                zip(db.snapshot_get_cols, fetched_branches[branches_count])
+            )["name"]
 
         if branches:
             return PartialBranches(
@@ -880,7 +948,11 @@ class Storage:
                 origin_visit = self.origin_visit_get_by(
                     visit_status.origin, visit_status.visit, db=db, cur=cur
                 )
-                assert origin_visit is not None
+                if origin_visit is None:
+                    raise StorageArgumentException(
+                        f"Unknown origin visit {visit_status.visit} "
+                        f"of origin {visit_status.origin}"
+                    )
 
                 origin_visit_status = attr.evolve(visit_status, type=origin_visit.type)
             else:
@@ -1147,6 +1219,7 @@ class Storage:
         limit: int = 50,
         regexp: bool = False,
         with_visit: bool = False,
+        visit_types: Optional[List[str]] = None,
         db=None,
         cur=None,
     ) -> PagedResult[Origin]:
@@ -1156,7 +1229,7 @@ class Storage:
         origins = []
         # Take one more origin so we can reuse it as the next page token if any
         for origin in db.origin_search(
-            url_pattern, offset, limit + 1, regexp, with_visit, cur
+            url_pattern, offset, limit + 1, regexp, with_visit, visit_types, cur
         ):
             row_d = dict(zip(db.origin_cols, origin))
             origins.append(Origin(url=row_d["url"]))
@@ -1231,13 +1304,14 @@ class Storage:
     ) -> None:
         metadata = list(metadata)
         self.journal_writer.raw_extrinsic_metadata_add(metadata)
-        counter = Counter[MetadataTargetType]()
+        counter = Counter[ExtendedObjectType]()
         for metadata_entry in metadata:
             authority_id = self._get_authority_id(metadata_entry.authority, db, cur)
             fetcher_id = self._get_fetcher_id(metadata_entry.fetcher, db, cur)
 
             db.raw_extrinsic_metadata_add(
-                type=metadata_entry.type.value,
+                id=metadata_entry.id,
+                type=metadata_entry.target.object_type.name.lower(),
                 target=str(metadata_entry.target),
                 discovery_date=metadata_entry.discovery_date,
                 authority_id=authority_id,
@@ -1253,20 +1327,19 @@ class Storage:
                 directory=map_optional(str, metadata_entry.directory),
                 cur=cur,
             )
-            counter[metadata_entry.type] += 1
+            counter[metadata_entry.target.object_type] += 1
 
         for (type, count) in counter.items():
             send_metric(
                 f"{type.value}_metadata:add",
                 count=count,
-                method_name=f"{type.value}_metadata_add",
+                method_name=f"{type.name.lower()}_metadata_add",
             )
 
     @db_transaction()
     def raw_extrinsic_metadata_get(
         self,
-        type: MetadataTargetType,
-        target: Union[str, SWHID],
+        target: ExtendedSWHID,
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
@@ -1274,19 +1347,6 @@ class Storage:
         db=None,
         cur=None,
     ) -> PagedResult[RawExtrinsicMetadata]:
-        if type == MetadataTargetType.ORIGIN:
-            if isinstance(target, SWHID):
-                raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with type='origin', "
-                    f"but provided target is a SWHID: {target!r}"
-                )
-        else:
-            if not isinstance(target, SWHID):
-                raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with type!='origin', "
-                    f"but provided target is not a SWHID: {target!r}"
-                )
-
         if page_token:
             (after_time, after_fetcher) = msgpack_loads(base64.b64decode(page_token))
             if after and after_time < after:
