@@ -264,7 +264,11 @@ alter table metadata_authority add primary key using index metadata_authority_pk
 
 
 -- raw_extrinsic_metadata
-create unique index concurrently raw_extrinsic_metadata_content_authority_date_fetcher on raw_extrinsic_metadata(target, authority_id, discovery_date, fetcher_id);
+
+create unique index concurrently raw_extrinsic_metadata_pkey on raw_extrinsic_metadata(id);
+alter table raw_extrinsic_metadata add primary key using index raw_extrinsic_metadata_pkey;
+
+create index concurrently raw_extrinsic_metadata_content_authority_date on raw_extrinsic_metadata(target, authority_id, discovery_date);
 
 \if :dbflavor_default
   alter table raw_extrinsic_metadata add constraint raw_extrinsic_metadata_authority_fkey foreign key (authority_id) references metadata_authority(id) not valid;
@@ -281,3 +285,9 @@ alter table object_counts add primary key using index object_counts_pkey;
 -- object_counts_bucketed
 create unique index concurrently object_counts_bucketed_pkey on object_counts_bucketed(line);
 alter table object_counts_bucketed add primary key using index object_counts_bucketed_pkey;
+
+-- extid
+
+-- used to query by (extid_type, extid) + to deduplicate the whole row
+create unique index concurrently on extid(extid_type, extid, target_type, target);
+create index concurrently on extid(target_type, target);
