@@ -5,7 +5,7 @@
 
 import multiprocessing.util
 
-from hypothesis import settings
+from hypothesis import HealthCheck, settings
 import pytest
 
 try:
@@ -19,6 +19,13 @@ from swh.model.model import BaseContent, Origin
 from swh.model.tests.generate_testdata import gen_contents, gen_origins
 from swh.storage.interface import StorageInterface
 
+# we use getattr here to keep mypy happy regardless hypothesis version
+function_scoped_fixture_check = (
+    [getattr(HealthCheck, "function_scoped_fixture")]
+    if hasattr(HealthCheck, "function_scoped_fixture")
+    else []
+)
+
 # define tests profile. Full documentation is at:
 # https://hypothesis.readthedocs.io/en/latest/settings.html#settings-profiles
 settings.register_profile("fast", max_examples=5, deadline=5000)
