@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -9,16 +9,17 @@ from typing import Tuple
 import attr
 
 from swh.model import from_disk
-from swh.model.hashutil import hash_to_bytes, hash_to_hex
-from swh.model.identifiers import parse_swhid
+from swh.model.hashutil import hash_to_bytes
+from swh.model.identifiers import CoreSWHID, ExtendedObjectType, ExtendedSWHID
+from swh.model.identifiers import ObjectType as SwhidObjectType
 from swh.model.model import (
     Content,
     Directory,
     DirectoryEntry,
+    ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
     ObjectType,
     Origin,
     OriginVisit,
@@ -312,7 +313,139 @@ class StorageData:
         extra_headers=(),
         synthetic=False,
     )
-    revisions: Tuple[Revision, ...] = (revision, revision2, revision3, revision4)
+    git_revisions: Tuple[Revision, ...] = (revision, revision2, revision3, revision4)
+
+    hg_revision = Revision(
+        id=hash_to_bytes("951c9503541e7beaf002d7aebf2abd1629084c68"),
+        message=b"hello",
+        author=Person(
+            name=b"Nicolas Dandrimont",
+            email=b"nicolas@example.com",
+            fullname=b"Nicolas Dandrimont <nicolas@example.com> ",
+        ),
+        date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1234567890, microseconds=0),
+            offset=120,
+            negative_utc=False,
+        ),
+        committer=Person(
+            name=b"St\xc3fano Zacchiroli",
+            email=b"stefano@example.com",
+            fullname=b"St\xc3fano Zacchiroli <stefano@example.com>",
+        ),
+        committer_date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1123456789, microseconds=0),
+            offset=120,
+            negative_utc=False,
+        ),
+        parents=(),
+        type=RevisionType.MERCURIAL,
+        directory=directory.id,
+        metadata={
+            "checksums": {"sha1": "tarball-sha1", "sha256": "tarball-sha256",},
+            "signed-off-by": "some-dude",
+            "node": "a316dfb434af2b451c1f393496b7eaeda343f543",
+        },
+        extra_headers=(),
+        synthetic=True,
+    )
+    hg_revision2 = Revision(
+        id=hash_to_bytes("df4afb063236300eb13b96a0d7fff03f7b7cbbaf"),
+        message=b"hello again",
+        author=Person(
+            name=b"Roberto Dicosmo",
+            email=b"roberto@example.com",
+            fullname=b"Roberto Dicosmo <roberto@example.com>",
+        ),
+        date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1234567843, microseconds=220000,),
+            offset=-720,
+            negative_utc=False,
+        ),
+        committer=Person(
+            name=b"tony", email=b"ar@dumont.fr", fullname=b"tony <ar@dumont.fr>",
+        ),
+        committer_date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1123456789, microseconds=220000,),
+            offset=0,
+            negative_utc=False,
+        ),
+        parents=tuple([hg_revision.id]),
+        type=RevisionType.MERCURIAL,
+        directory=directory2.id,
+        metadata=None,
+        extra_headers=(
+            (b"node", hash_to_bytes("fa1b7c84a9b40605b67653700f268349a6d6aca1")),
+        ),
+        synthetic=False,
+    )
+    hg_revision3 = Revision(
+        id=hash_to_bytes("84d8e7081b47ebb88cad9fa1f25de5f330872a37"),
+        message=b"a simple revision with no parents this time",
+        author=Person(
+            name=b"Roberto Dicosmo",
+            email=b"roberto@example.com",
+            fullname=b"Roberto Dicosmo <roberto@example.com>",
+        ),
+        date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1234567843, microseconds=220000,),
+            offset=-720,
+            negative_utc=False,
+        ),
+        committer=Person(
+            name=b"tony", email=b"ar@dumont.fr", fullname=b"tony <ar@dumont.fr>",
+        ),
+        committer_date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1127351742, microseconds=220000,),
+            offset=0,
+            negative_utc=False,
+        ),
+        parents=tuple([hg_revision.id, hg_revision2.id]),
+        type=RevisionType.MERCURIAL,
+        directory=directory2.id,
+        metadata=None,
+        extra_headers=(
+            (b"node", hash_to_bytes("7f294a01c49065a90b3fe8b4ad49f08ce9656ef6")),
+        ),
+        synthetic=True,
+    )
+    hg_revision4 = Revision(
+        id=hash_to_bytes("42070a39e5387e9b99bb3d83674e3a4a1ff39b69"),
+        message=b"parent of self.revision2",
+        author=Person(
+            name=b"me", email=b"me@soft.heri", fullname=b"me <me@soft.heri>",
+        ),
+        date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1234567843, microseconds=220000,),
+            offset=-720,
+            negative_utc=False,
+        ),
+        committer=Person(
+            name=b"committer-dude",
+            email=b"committer@dude.com",
+            fullname=b"committer-dude <committer@dude.com>",
+        ),
+        committer_date=TimestampWithTimezone(
+            timestamp=Timestamp(seconds=1244567843, microseconds=220000,),
+            offset=-720,
+            negative_utc=False,
+        ),
+        parents=tuple([hg_revision3.id]),
+        type=RevisionType.MERCURIAL,
+        directory=directory.id,
+        metadata=None,
+        extra_headers=(
+            (b"node", hash_to_bytes("f4160af0485c85823d9e829bae2c00b00a2e6297")),
+        ),
+        synthetic=False,
+    )
+    hg_revisions: Tuple[Revision, ...] = (
+        hg_revision,
+        hg_revision2,
+        hg_revision3,
+        hg_revision4,
+    )
+    revisions: Tuple[Revision, ...] = git_revisions + hg_revisions
 
     origins: Tuple[Origin, ...] = (
         Origin(url="https://github.com/user1/repo1"),
@@ -463,8 +596,9 @@ class StorageData:
     snapshots: Tuple[Snapshot, ...] = (snapshot, empty_snapshot, complete_snapshot)
 
     content_metadata1 = RawExtrinsicMetadata(
-        type=MetadataTargetType.CONTENT,
-        target=parse_swhid(f"swh:1:cnt:{hash_to_hex(content.sha1_git)}"),
+        target=ExtendedSWHID(
+            object_type=ExtendedObjectType.CONTENT, object_id=content.sha1_git
+        ),
         origin=origin.url,
         discovery_date=datetime.datetime(
             2015, 1, 1, 21, 0, 0, tzinfo=datetime.timezone.utc
@@ -475,8 +609,9 @@ class StorageData:
         metadata=b'{"foo": "bar"}',
     )
     content_metadata2 = RawExtrinsicMetadata(
-        type=MetadataTargetType.CONTENT,
-        target=parse_swhid(f"swh:1:cnt:{hash_to_hex(content.sha1_git)}"),
+        target=ExtendedSWHID(
+            object_type=ExtendedObjectType.CONTENT, object_id=content.sha1_git
+        ),
         origin=origin2.url,
         discovery_date=datetime.datetime(
             2017, 1, 1, 22, 0, 0, tzinfo=datetime.timezone.utc
@@ -487,8 +622,9 @@ class StorageData:
         metadata=b"foo: bar",
     )
     content_metadata3 = RawExtrinsicMetadata(
-        type=MetadataTargetType.CONTENT,
-        target=parse_swhid(f"swh:1:cnt:{hash_to_hex(content.sha1_git)}"),
+        target=ExtendedSWHID(
+            object_type=ExtendedObjectType.CONTENT, object_id=content.sha1_git
+        ),
         discovery_date=datetime.datetime(
             2017, 1, 1, 22, 0, 0, tzinfo=datetime.timezone.utc
         ),
@@ -498,10 +634,10 @@ class StorageData:
         metadata=b"foo: bar",
         origin=origin.url,
         visit=42,
-        snapshot=parse_swhid(f"swh:1:snp:{hash_to_hex(snapshot.id)}"),
-        release=parse_swhid(f"swh:1:rel:{hash_to_hex(release.id)}"),
-        revision=parse_swhid(f"swh:1:rev:{hash_to_hex(revision.id)}"),
-        directory=parse_swhid(f"swh:1:dir:{hash_to_hex(directory.id)}"),
+        snapshot=snapshot.swhid(),
+        release=release.swhid(),
+        revision=revision.swhid(),
+        directory=directory.swhid(),
         path=b"/foo/bar",
     )
 
@@ -512,8 +648,7 @@ class StorageData:
     )
 
     origin_metadata1 = RawExtrinsicMetadata(
-        type=MetadataTargetType.ORIGIN,
-        target=origin.url,
+        target=Origin(origin.url).swhid(),
         discovery_date=datetime.datetime(
             2015, 1, 1, 21, 0, 0, tzinfo=datetime.timezone.utc
         ),
@@ -523,8 +658,7 @@ class StorageData:
         metadata=b'{"foo": "bar"}',
     )
     origin_metadata2 = RawExtrinsicMetadata(
-        type=MetadataTargetType.ORIGIN,
-        target=origin.url,
+        target=Origin(origin.url).swhid(),
         discovery_date=datetime.datetime(
             2017, 1, 1, 22, 0, 0, tzinfo=datetime.timezone.utc
         ),
@@ -534,8 +668,7 @@ class StorageData:
         metadata=b"foo: bar",
     )
     origin_metadata3 = RawExtrinsicMetadata(
-        type=MetadataTargetType.ORIGIN,
-        target=origin.url,
+        target=Origin(origin.url).swhid(),
         discovery_date=datetime.datetime(
             2017, 1, 1, 22, 0, 0, tzinfo=datetime.timezone.utc
         ),
@@ -550,3 +683,29 @@ class StorageData:
         origin_metadata2,
         origin_metadata3,
     )
+
+    extid1 = ExtID(
+        target=CoreSWHID(object_type=SwhidObjectType.REVISION, object_id=revision.id),
+        extid_type="git",
+        extid=revision.id,
+    )
+
+    extid2 = ExtID(
+        target=CoreSWHID(
+            object_type=SwhidObjectType.REVISION, object_id=hg_revision.id
+        ),
+        extid_type="mercurial",
+        extid=hash_to_bytes("a316dfb434af2b451c1f393496b7eaeda343f543"),
+    )
+
+    extid3 = ExtID(
+        target=CoreSWHID(object_type=SwhidObjectType.DIRECTORY, object_id=directory.id),
+        extid_type="directory",
+        extid=b"something",
+    )
+
+    extids: Tuple[ExtID, ...] = (
+        extid1,
+        extid2,
+        extid3,
+    )
